from pathlib import Path
import random, string
try:
    import ujson as json
except ModuleNotFoundError:
    import json
from httpx import AsyncClient


# 载入词库(这个词库有点涩)
AnimeThesaurus = json.load(open(Path(__file__).parent / 'resource' / 'data.json', 'r', encoding='utf8'))

# hello之类的回复
hello__reply = [
    'ʕ  •ᴥ•ʔ ?',
]

# 从字典里返还消息, 借鉴(抄)的zhenxun-bot
async def get_chat_result(text: str):
    if len(text) < 7:
        keys = AnimeThesaurus.keys()
        for key in keys:
            if text.find(key) != -1:
                return random.choice(AnimeThesaurus[key])

headers = {
    'referer': 'https://www.ownthink.com/',
    'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/109.0.0.0 Safari/537.36'
}

# 从ownthink_api拿到消息
async def get_reply(msg, NICKNAME):
    for i in string.punctuation:
        msg = msg.replace(i, ' ')

    url = f'https://api.ownthink.com/bot'
    params = {
        'appid': 'xiaosi',
        'userid': 'user',
        'spoken': msg.strip(),
    }

    async with AsyncClient() as client:
        try:
            response = await client.get(url=url, params=params, headers=headers)
            if response.json()['data']['type'] == 5000:
                res = response.json()['data']['info']['text'].replace('小思', NICKNAME)
                return res
            else:
                return 'ʕ  •ᴥ•ʔ……'
        except Exception as e:
            return repr(e)



import random
from telegram import Update
from telegram.ext import (
    ContextTypes,
    MessageHandler,
    filters
)

from remilia.log import logger
from remilia.config import NICKNAME
from .utils import (
    get_reply,
    get_chat_result,
    hello__reply
)
from .looklike import Look


def run(application):
    chat_handler = MessageHandler(
        filters.ChatType.PRIVATE & filters.TEXT & (~filters.COMMAND),
        chat
    )
    application.add_handler(chat_handler)

    groupchat_handler = MessageHandler(
        filters.ChatType.GROUPS & filters.TEXT & (~filters.COMMAND),
        groupchat
    )
    application.add_handler(groupchat_handler)


async def chat(update: Update, context: ContextTypes.DEFAULT_TYPE):
    logger.info(f'[{update.message.chat.type.upper()}]({update.message.chat_id}) {context._user_id}: {update.message.text}')
    msg = update.message.text
    content = await reply(msg, NICKNAME[0])
    await update.message.chat.send_message(content)


async def groupchat(update: Update, context: ContextTypes.DEFAULT_TYPE):
    logger.info(f'[{update.message.chat.type.upper()}]({update.message.chat_id}) {context._user_id}: {update.message.text}')
    msg = update.message.text
    for i in NICKNAME:
        if msg.startswith(i):
            msg = msg.replace(i, '', 1)
            content = await reply(msg, NICKNAME[0])
            await update.message.chat.send_message(content)


async def reply(msg: str, NICKNAME: str):
    if (not msg) or msg.isspace() or msg in [
        "你好啊",
        "你好",
        "在吗",
        "在嗎",
        "在不在",
        "您好",
        "您好啊",
        "在",
    ]:
        return random.choice(hello__reply)
    if msg.startswith('你看我像'):
        return Look.like()
    # 从字典里获取结果
    result = await get_chat_result(msg)
    # 如果词库没有结果，则调用对话api获取回复
    if not result:
        content = await get_reply(msg, NICKNAME)
        return content
    return result



from subprocess import Popen, PIPE
from html import unescape


def help() -> str:
    return (
        '调用系统命令行\n'
        '⚠危险操作, 谨慎使用!\n\n'
        '/cmd {命令}\n'
        'For example:\n'
        '/cmd echo "Hello World"'
    )


def shell(opt: str) -> str:

    if not opt:
        return '发送 /cmd -h 获取帮助'

    if opt == '-h':
        return help()

    content = Popen(
        unescape(opt.strip()),
        stdin=PIPE,
        stdout=PIPE,
        stderr=PIPE,
        shell=True,
        universal_newlines=True
    ).communicate()

    if content == ('',''):
        msg = '执行完毕, 没有任何输出呢~'
    elif not content[1]:
        msg = f'stdout:\n{content[0]}\n>执行完毕'
    elif not content[0]:
        msg = f'stderr:\n{content[1]}'
    else:
        msg = f'stdout:\n{content[0]}\nstderr:\n{content[1]}\n>执行完毕'
    return msg


